@@ -8,7 +8,7 @@ use RayApp ();
 use IO::ScalarArray ();
 
 sub handler {
-	my $filename;
+	my ($filename, $uri);
 	# print STDERR "request_uri [$ENV{REQUEST_URI}]\n";
 	# print STDERR "script_name [$ENV{SCRIPT_NAME}]\n";
 	# print STDERR "path_info [$ENV{PATH_INFO}]\n";
@@ -17,6 +17,7 @@ sub handler {
 	# The URI can come as an argument on command line ...
 	if (@ARGV) {
 		$filename = shift @ARGV;
+		$uri = "file:$filename";
 	}
 
 	# ... in PATH_INFO in CGI / rayapp_cgi_wrapper ScriptAlias
@@ -26,6 +27,7 @@ sub handler {
 		$filename = $ENV{RAYAPP_DIRECTORY};
 		$filename .= $ENV{PATH_INFO} if defined $ENV{PATH_INFO};
 		# $ENV{SCRIPT_NAME} = $ENV{REQUEST_URI};
+		$uri = $ENV{REQUEST_URI};
 		delete $ENV{PATH_INFO};
 		$ENV{PATH_TRANSLATED} = $filename;
 
@@ -43,13 +45,17 @@ sub handler {
 	# ... maybe even PATH_TRANSLATED would work
 	elsif (defined $ENV{PATH_TRANSLATED}) {
 		$filename = $ENV{PATH_TRANSLATED};
+		$uri = "file:$filename";
 	}
 
 	# ... or the URI is specified as $0, by running the .pl script
 	else {
 		$filename = $0;
+		$uri = "file:$filename";
 	}
 
+	local $0 = "$0 uri [$uri] filename [$filename]";
+
 	# print STDERR "Filename [$filename]\n";
 
 	# If the URI is a directory, we shall use RAYAPP_DIRECTORY_INDEX
@@ -65,11 +71,11 @@ sub handler {
 		local *FILE;
 		open FILE, $filename or do {
 			print "Status: 403\nContent-Type: text/plain\n\n";
-			print "Failed to load data for [$filename]\n";
+			print "Failed to load data for [$uri]\n";
 			print STDERR "RayApp::CGI: filename [$filename] error $!\n";
 			exit;
 		};
-		print STDERR "RayApp::CGI: serving local file [$filename]\n";
+		# print STDERR "RayApp::CGI: serving local file [$filename]\n";
 		local $/ = undef;
 		print "Status: 200\n";
 		if ($filename =~ /\.html$/) {
@@ -90,20 +96,20 @@ sub handler {
 	my $ext;
 	$stripped_filename =~ s!\.([^./]+)$!! and $ext = $1;
 
-	my ($xml, $dom, @params);
+	my ($xml, $dom, $dsd_filename, $application, @params, @stylesheets);
 	if (-f $stripped_filename . '.xml') {
 
 		$filename = $stripped_filename . '.xml';
 		$xml = $rayapp->load_xml($filename) or do {
 			print "Status: 500\nContent-Type: text/plain\n\n";
-			print "Failed to load data for [$filename]\n";
+			print "Failed to load data for [$uri]\n";
 			print STDERR "RayApp::CGI: filename [$filename] error ", $rayapp->errstr, "\n";
                         exit;
                 };
 		$dom = $xml->xmldom;
+		@stylesheets = $xml->find_stylesheets($ext);
 	} else {
 
-		my $dsd_filename;
 		if (defined $ext) {
 			$dsd_filename = $stripped_filename . '.dsd';
 		} else {
@@ -111,7 +117,7 @@ sub handler {
 		}
 		if (not -f $dsd_filename) {
 			print "Status: 404\nContent-Type: text/plain\n\n";
-			# print "Failed to load output specification for [$filename]\n";
+			# print "Failed to load output specification for [$uri]\n";
 			print "The requested URL was not found on this server.\n";
 			print STDERR "RayApp::CGI: filename [$filename] no DSD found\n";
                         exit;
@@ -120,12 +126,12 @@ sub handler {
 		$xml = $rayapp->load_dsd($dsd_filename);
 		if (not defined $xml) {
 			print "Status: 500\nContent-Type: text/plain\n\n";
-			print "Failed to load output specification for [$filename]\n";
+			print "Failed to load output specification for [$uri]\n";
 			print STDERR "RayApp::CGI: filename [$filename] DSD [$dsd_filename] error ", $rayapp->errstr, "\n";
                         exit;
 		}
 
-		my $application = $xml->application_name;
+		$application = $xml->application_name;
 		if (not defined $application) {
 			for ('.mpl', '.pl') {
 				if (-f $stripped_filename . $_) {
@@ -135,12 +141,14 @@ sub handler {
 			}
 			if (not defined $application) {
 				print "Status: 404\nContent-Type: text/plain\n\n";
-				print "Failed to load output specification for [$filename]\n";
+				print "Failed to load output specification for [$uri]\n";
 				print STDERR "RayApp::CGI: filename [$filename] DSD [$dsd_filename] no application found\n";
 				exit;
 			}
 		}
-
+	}
+	if (defined $application
+		or (@stylesheets and $ENV{'RAYAPP_STYLE_STATIC_PARAMS'})) {
 		my $input_module = $ENV{'RAYAPP_INPUT_MODULE'};
 		if (defined $input_module) {
 			my $package = __PACKAGE__;
@@ -149,24 +157,29 @@ sub handler {
 			eval qq!#line $line "$package"\nuse $input_module;!;
 			if ($@) {
 				print "Status: 500\nContent-Type: text/plain\n\n";
-				print "Failed to load input module for [$filename]\n";
-				print STDERR "RayApp::CGI: filename [$filename] DSD [$dsd_filename] input module [$input_module] failed:\n$@\n";
+				print "Failed to load input module for [$uri]\n";
+				$@ =~ s/\n+$//; $@ =~ s/(^|\n)/$1  /g;
+				print STDERR "RayApp::CGI: filename [$filename] DSD [$dsd_filename] failed to load input module [$input_module]:\n$@\n";
 				exit;
 			}
 
 			my $handler = "${input_module}::handler";
 			{
 				no strict;
-				eval { @params = &{ $handler }($xml); };
+				eval { @params = &{ $handler }(
+					( defined $application ? $xml : () )
+				); };
 			}
 			if ($@) {
 				print "Status: 500\nContent-Type: text/plain\n\n";
-				print "Failed to run input module for [$filename]\n";
-				print STDERR "RayApp::CGI: filename [$filename] DSD [$dsd_filename] input module [$input_module]\n$@\n";
+				print "Failed to run input module for [$uri]\n";
+				$@ =~ s/\n+$//; $@ =~ s/(^|\n)/$1  /g;
+				print STDERR "RayApp::CGI: filename [$filename] DSD [$dsd_filename] failed to run input module [$input_module]:\n$@\n";
 				exit;	
 			}
 		}
-
+	}
+	if ($application) {
 		my @stdout_data;
 		my $err;
 		my $data;
@@ -192,14 +205,14 @@ sub handler {
 
 		if (defined $err) {
 			print "Status: 500\nContent-Type: text/plain\n\n";
-			print "Failed to run application for [$filename]\n";
+			print "Failed to run application for [$uri]\n";
 			print STDERR "RayApp::CGI: filename [$filename] DSD [$dsd_filename] application [$application]\n$err\n";
 			exit;
 		}
 
 		if (not defined $data) {
 			print "Status: 500\nContent-Type: text/plain\n\n";
-			print "Failed to run application for [$filename]\n";
+			print "Failed to run application for [$uri]\n";
 			print STDERR "RayApp::CGI: filename [$filename] DSD [$dsd_filename] application [$application] returned undef\n";
 			exit;
 		}
@@ -207,7 +220,7 @@ sub handler {
 		if (not ref $data) {
 			if ($data eq '500') {
 				print "Status: 500\nContent-Type: text/plain\n\n";
-				print "Failed to run application for [$filename]\n";
+				print "Failed to run application for [$uri]\n";
 				print STDERR "RayApp::CGI: filename [$filename] DSD [$dsd_filename] application [$application] returned 500: ", $rayapp->errstr, "\n";
 				exit;	
 			}
@@ -225,13 +238,13 @@ sub handler {
 		if (not defined $dom
 			or defined $xml->errstr) {
 			print "Status: 500\nContent-Type: text/plain\n\n";
-			print "Failed serialize output data for [$filename]\n";
+			print "Failed serialize output data for [$uri]\n";
 			print STDERR "RayApp::CGI: filename [$filename] DSD [$dsd_filename] ", $xml->errstr, "\n";
 			exit;	
 		}
+		@stylesheets = $xml->find_stylesheets($ext);
 	}
 
-	my @stylesheets = $xml->find_stylesheets($ext);
 	my %stylesheets_params;
 	if (@stylesheets
 		or defined $ENV{'HTTP_X_RAYAPP_FRONTEND_URI'}) {
@@ -243,8 +256,9 @@ sub handler {
 			eval qq!#line $line "$package"\nuse $style_param_module;!;
 			if ($@) {
 				print "Status: 500\nContent-Type: text/plain\n\n";
-				print "Failed to load style param module for [$filename]\n";
-				print STDERR "RayApp::CGI: filename [$filename] style param module [$style_param_module]\n$@\n";
+				print "Failed to load style param module for [$uri]\n";
+				$@ =~ s/\n+$//; $@ =~ s/(^|\n)/$1  /g;
+				print STDERR "RayApp::CGI: filename [$filename] failed to load style param module [$style_param_module]:\n$@\n";
 				exit;	
 			}
 
@@ -257,8 +271,9 @@ sub handler {
 			}
 			if ($@) {
 				print "Status: 500\nContent-Type: text/plain\n\n";
-				print "Failed to run style param module for [$filename]\n";
-				print STDERR "RayApp::CGI: filename [$filename] style param module [$style_param_module]\n$@\n";
+				print "Failed to run style param module for [$uri]\n";
+				$@ =~ s/\n+$//; $@ =~ s/(^|\n)/$1  /g;
+				print STDERR "RayApp::CGI: filename [$filename] failed to run style param module [$style_param_module]:\n$@\n";
 				exit;	
 			}
 		}
@@ -282,13 +297,16 @@ sub handler {
 		);
 		if (not defined $output) {
 			print "Status: 500\nContent-Type: text/plain\n\n";
-			print "Failed to style output for [$filename]\n";
-			print STDERR "RayApp::CGI: filename [$filename] style error ", $xml->errstr, "\n";
+			print "Failed to style output for [$uri]\n";
+			my $error = $xml->errstr;
+			$error =~ s/\n+$//;
+			print STDERR "RayApp::CGI: filename [$filename] style error:\n$error\n";
 			exit;	
 		}
 	} else {
 		my $i = 1;
 		for my $k (keys %stylesheets_params) {
+			next if not defined $stylesheets_params{$k};
 			my $data = "$k:$stylesheets_params{$k}";
 			$data =~ s/([^a-zA-Z0-9])/ sprintf "&#x%x;", ord $1 /ge;
 			push @headers_out, "X-RayApp-Style-Param-$i: $data\n";
@@ -326,7 +344,7 @@ sub handler {
 		unless ($processor =~ s/%IN/ $in->filename() /ge
 			and $processor =~ s/%OUT/ $out->filename() /ge) {
 			print "Status: 500\nContent-Type: text/plain\n\n";
-			print "Failed to generate PDF for [$filename]\n";
+			print "Failed to generate PDF for [$uri]\n";
 			print STDERR "RayApp::CGI: filename [$filename] processor [$processor] should have both %IN and %OUT\n";
 			exit;	
 		}
